;; futures creates n threads for each expression given (as an argument)
(defmacro futures
  [n & exprs]
  (vec (for [_ (range n)
             expr exprs]
         `(future ~expr))))

;; wait-futures will run futures and block until all the values are realized
(defmacro wait-futures
  [& args]
  `(doseq [f# (futures ~@args)]
     @f#))

;; examples
;; (defn one-sec [] (Thread/sleep 1000) (println "one"))
;; (defn three-and-done [] (Thread/sleep 3000) (println "done"))
;; (time (wait-futures 3 (three-and-done)))
;; (time (wait-futures 3 (one-sec) (three-and-done)))
;; (dotimes [_ 4] (one-sec))
